; DagreD3Gadget - Include

; -> https://github.com/cpettitt/dagre/wiki

EnableExplicit

Enumeration 1
	#PB_DagreD3_Border
EndEnumeration

OpenLibrary(#PB_Any, "userlibraries/d3/d3.js")
OpenLibrary(#PB_Any, "userlibraries/graphlib-dot/graphlib-dot.min.js")
OpenLibrary(#PB_Any, "userlibraries/dagre-d3/dagre-d3.js")

Procedure DagreD3Gadget_BindGadgetEvent(Gadget, Callback, EventType = #PB_All)
	
	! var selector = $("#g_" + Gadget);
	
	If EventType = #PB_All
		EventType = #PB_EventType_LeftClick
	EndIf
	
	!	if (cCommon.CheckFlag(EventType, cConstants.PB_EventType_LeftClick)) { 
	!		var inner = selector.data("inner");
	! 	var selections = inner.selectAll("g.node");
	!		selections.on('click', function (a,b,c,d,e,f,g) { 
	!			Callback(a);
	!		});
	! }
	
EndProcedure
RegisterUserGadgetProcedure("DagreD3Gadget", "BindGadgetEvent", @DagreD3Gadget_BindGadgetEvent())

Procedure DagreD3Gadget_ClearGadgetItems(Gadget)
	
	! var selector = $("#g_" + Gadget);
	!	var inner = selector.data("inner");
	! $(inner[0]).empty();
	
EndProcedure
RegisterUserGadgetProcedure("DagreD3Gadget", "ClearGadgetItems", @DagreD3Gadget_ClearGadgetItems())

; ------------------------

Procedure DagreD3Gadget_SetNode(Gadget, NodeText.s, LabelText.s="", Width=120)
	! var selector = $("#g_" + Gadget);
	! var graph = selector.data("graph");
	! if (LabelText=="") LabelText = NodeText;
	! return graph.setNode(NodeText, { label: LabelText, width: Width, clusterLabelPos: 'top' });
EndProcedure

Procedure DagreD3Gadget_SetEdge(Gadget, FromNode.s, ToNode.s, LabelText.s="")
	! var selector = $("#g_" + Gadget);
	! var graph = selector.data("graph");
	! return graph.setEdge(FromNode, ToNode, { label: LabelText, lineInterpolate: 'monotone'});
EndProcedure

Procedure DagreD3Gadget_SetParent(Gadget, ChildNode.s, ParentNode.s)
	! var selector = $("#g_" + Gadget);
	! var graph = selector.data("graph");
	! return graph.setParent(ChildNode, ParentNode);
EndProcedure

Procedure.s DagreD3Gadget_GetParent(Gadget, ChildNode.s)
	! var selector = $("#g_" + Gadget);
	! var graph = selector.data("graph");
	! // Set some general styles
	! graph.nodes().forEach(function(v) {
	!   var node = graph.node(v);
	!   var p = $(node).parent();
	!		debugger;
	! });
EndProcedure

Procedure DagreD3Gadget_Render(Gadget)
	
	! var selector = $("#g_" + Gadget);
	
	! var graph = selector.data("graph");
	!	var inner = selector.data("inner");
	! var svg   = selector.data("svg");
	! var zoom  = selector.data("zoom");
	
	!	// Create the renderer
	!	var render = dagreD3.render();
	
	!	// Run the renderer. This is what draws the final graph.
	!	render(inner, graph);
	

	! var newWidth  = graph.graph().width  + 10;
	! var newHeight = graph.graph().height + 10;
	
	! cGadget.ResizeGadget(Gadget, cConstants.PB_Ignore, cConstants.PB_Ignore, newWidth, newHeight);
	
EndProcedure

Procedure DagreD3Gadget_ToDot(Gadget, dot.s)
	
	! var selector = $("#g_" + Gadget);
	
	! var graph = selector.data("graph");
	!	var inner = selector.data("inner");
	! var svg   = selector.data("svg");
	! var zoom  = selector.data("zoom");
	
	!   // Parse the DOT syntax into a graphlib object.
	!   var parsedDot = graphlibDot.read(dot);
	!   // Render the graphlib object using d3.
	!   var renderer = new dagreD3.render();
	!   renderer(inner, parsedDot);
	
; 	!   // Optional - resize the SVG element based on the contents.
; 	!   var svg = document.querySelector('#graphContainer');
; 	!   var bbox = svg.getBBox();
; 	!   svg.style.width = bbox.width + 40.0 + "px";
; 	!   svg.style.height = bbox.height + 40.0 + "px";	
	
EndProcedure


Procedure DagreD3Gadget(Gadget, x, y, Width, Height, Flags = 0)
	
	! Gadget = cCommon.SetGadgetAndStuff(Gadget);

	! var selector = $("<svg><g /></svg>");

	! selector.attr("id", "g_" + Gadget);
	
	!	if (cCommon.CheckFlag(Flags, PB_DagreD3_Border)) {
	!		selector.css("border", "1px solid black");
	! }

	! cCommon.CurrentContainer.append(selector);

	! cCommon.SetGadgetPosition(selector, x, y, Width, Height);
	
	!	// create graph
	!	var graph = new dagreD3.graphlib.Graph({
	!		multigraph: false,
	!		compound: true
	!	}).setGraph({
	!		rankdir: "LR", // Direction for rank nodes. Can be TB, BT, LR, or RL, where T = top, B = bottom, L = left, and R = right.
	!		edgesep: 10,
	!		nodesep: 10,
	!		marginx: 10,
	!		marginy: 10
	!	});	
	
	! var svg = d3.select("#g_" + Gadget);
	! var inner = d3.select("#g_" + Gadget + " g");
	! // Set up zoom support
	! var zoom = d3.behavior.zoom().on("zoom", function() {
	! 	inner.attr("transform", "translate(" + d3.event.translate + ")scale(" + d3.event.scale + ")");
	!	});
	! svg.call(zoom);	
	
	!	selector.data("svg", svg);
	!	selector.data("inner", inner);
	!	selector.data("graph", graph);
	!	selector.data("zoom", zoom);
	
	!	selector.data("GadgetType", "DagreD3Gadget");
	
	! return Gadget;	
	
EndProcedure
; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; CursorPosition = 36
; FirstLine = 3
; Folding = F7
; EnableXP