EnableExplicit

OpenLibrary(#PB_Any, "userlibraries/select2/select2.min.css")
OpenLibrary(#PB_Any, "userlibraries/select2/select2.min.js")

Enumeration
	
	; Original:
	; #PB_EventType_Change   : The current selection of the text in the edit field changed.
	; #PB_EventType_Focus    : The edit field received the keyboard focus (editable ComboBox only).
	; #PB_EventType_LostFocus: The edit field lost the keyboard focus (editable ComboBox only).	
	
	#PB_EventType_Close
	#PB_EventType_Closing
	#PB_EventType_Open
	#PB_EventType_Opening
	#PB_EventType_Selecting
	#PB_EventType_Unselected
	#PB_EventType_Unselecting
	
EndEnumeration

; AddGadgetItem() / RemoveGadgetItem()

Procedure Select2Gadget_AddGadgetItem(Gadget, Position, Text, ImageID = -1, Flags = 0)
	
	! var selector = $("#g_" + Gadget);
	!	var oArray = selector.children().toArray();
	!	if (oArray === null) oArray = [];
	!	if (Position === -1) Position = oArray.length;
	!	if (Position > oArray.length) Position = oArray.length;
	
	! var newOption = $("<option />");
	! if (ImageID != -1) newOption.attr("ImageID", ImageID);
	!	newOption.text(Text);

	!	oArray.splice(Position, 0, newOption);
	
	!	selector.empty();
	! selector.append(oArray);
	
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "AddGadgetItem", @Select2Gadget_AddGadgetItem())

Procedure Select2Gadget_RemoveGadgetItem(Gadget, Position)
	
	! var selector = $("#g_" + Gadget);
	!	var oArray = selector.children().toArray();
	!	if (oArray === null || oArray.length === 0) return;
	!	oArray.splice(Position, 1);
	!	selector.empty();
	!	selector.append(oArray);
	
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "RemoveGadgetItem", @Select2Gadget_RemoveGadgetItem())

; Set-/GetGadgetState()

Procedure Select2Gadget_SetGadgetState(Gadget, State)
	
	!	var selector = $("#g_" + Gadget);
	!	selector.find("option").removeAttr("selected");
	! 
	!	if (State === -1) {
	! 	selector.val("").trigger("change");
	!		return;
	!	}
	
	! var SelectedVal = selector.find("option").eq(State).val();
	! selector.val(SelectedVal).trigger("change");
	
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "SetGadgetState", @Select2Gadget_SetGadgetState())

Procedure Select2Gadget_GetGadgetState(Gadget)
	!	var selector = $("#g_" + Gadget);
	! return selector.prop("selectedIndex");
	! // return selector.val();
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "GetGadgetState", @Select2Gadget_GetGadgetState())

; Set-/GetGadgetText()

Procedure Select2Gadget_SetGadgetText(Gadget, Text)
	!	var selector = $("#g_" + Gadget);
	! selector.val(Text);
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "SetGadgetText", @Select2Gadget_SetGadgetText())

Procedure Select2Gadget_GetGadgetText(Gadget)
	!	var selector = $("#g_" + Gadget);
	! return selector.val();
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "GetGadgetText", @Select2Gadget_GetGadgetText())

; ---

Procedure Select2Gadget_BindGadgetEvent(Gadget, Callback, EventType)
	
	!	var selector = $("#g_" + Gadget);
	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("BindGadgetEvent");
	
	If EventType = #PB_All
		
		EventType = #PB_EventType_Change
		
	EndIf
	
	!	var cb = Callback;
	
	!	if (cCommon.CheckFlag(EventType, cConstants.PB_EventType_Change)) selector.data("onChange", cb);
	
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "BindGadgetEvent", @Select2Gadget_BindGadgetEvent())

Procedure Select2Gadget_FreeGadget(Gadget)
	!	var selector = $("#" + Gadget);
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("FreeGadget");	
	! selector.select2('destroy');
	! selector.remove();
EndProcedure
RegisterUserGadgetProcedure("Select2Gadget", "FreeGadget", @Select2Gadget_FreeGadget())

; ---------------------------------

Procedure Select2Gadget(Gadget, x, y, Width, Height, Flags = 0)
	
	!	var Gadget = cCommon.SetGadgetAndStuff(Gadget);
	
	!	var selector = $("<select />");
	
	!	selector.attr("id", "g_" + Gadget);
	
	!	cCommon.SetGadgetPosition(selector, x, y, Width, Height);
	
	!	selector.data("GadgetType", "Select2Gadget");
	
	!	cCommon.CurrentContainer.append(selector);
	
	! function formatResult (item) {
	
	!		var ImageID = $(item.element).attr("ImageID");
	!		if (!ImageID) { cDebugger.Debug("!") ; return item.text; }
	!		var img = cCommon.P2W_ImageList[ImageID];
	
	!		return $("<span><img src='" + img.src + "' style='vertical-align:middle;width:" + img.width + "px;height:" + img.height + "px' /> " + item.text + "</span>");
	
	!	}
	
	!	selector.select2({
	!   templateResult: formatResult,
	!   templateSelection: formatResult,
	!		data: [] 
	!	});	
	
	; Events
	
	! selector.on('change', function (evt) { // Triggered whenever an option is selected or removed.
	!		cb = selector.data("onChange");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Change;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:close', function (evt) { // Triggered whenever the dropdown is closed.
	!		cb = selector.data("onClose");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Close;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:closing', function (evt) { // Triggered before the dropdown is closed. This event can be prevented.
	!		cb = selector.data("onClosing");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Closing;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:open', function (evt) { // Triggered whenever the dropdown is opened.
	!		cb = selector.data("onOpen");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Open;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:opening', function (evt) { // Triggered before the dropdown is opened. This event can be prevented.
	!		cb = selector.data("onOpening");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Opening;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:select', function (evt) { // Triggered whenever a result is selected.
	!		cb = selector.data("onSelect");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Select;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:selecting', function (evt) { // Triggered before a result is selected. This event can be prevented.
	!		cb = selector.data("onSelecting");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Selecting;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:unselect', function (evt) { // Triggered whenever a selection is removed.
	!		cb = selector.data("onUnSelected");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Unselected;
	!			cb();
	!		}	
	! });
	
	! selector.on('select2:unselecting', function (evt) { // Triggered before a selection is removed. This event can be prevented.	
	!		cb = selector.data("onUnSelecting");
	!		if (typeof cb === "function") {
	!			cEvent.eventGadget = Gadget;
	!			cEvent.eventType = cConstants.PB_EventType_Unselecting;
	!			cb();
	!		}	
	! });
	
	!	return Gadget;
	
EndProcedure

; IDE Options = PureBasic 5.50 (Windows - x86)
; CursorPosition = 143
; Folding = Aw
; EnableXP