﻿
; https://olifolkerd.github.io/tabulator/

EnableExplicit

OpenLibrary(#PB_Any, "userlibraries/Tabulator/tabulator.css")
OpenLibrary(#PB_Any, "userlibraries/Tabulator/tabulator.js")

Enumeration
	
	; Set- / GetGadgetAttribute
	
	#TabulatorGadgetData
	#TabulatorGadgetColumns
	#TabulatorGadgetOptions
	#TabulatorGadgetGroupBy
	#TabulatorGadgetFitColumns
	
EndEnumeration

Structure sTabulatorGadgetColumn
	title.s           ; Required This is the title that will be displayed in the header for this column
	field.s						; Required (not required in icon/button columns) this is the key for this column in the data array
 	align.s 					; sets the text alignment for this column (left|center|right)
	width.i						; sets the width of this column (if not set the system will determine the best)
	sortable.i				; determines if the user can sort data by this column (see Sorting Data for more details)
	sorter.s					; determines how to sort data in this column (see Sorting Data for more details)
	formatter.s				; set how you would like the data to be formatted (see Formatting Data for more details)
	onClick.i					; callback for when user clicks on a cell in this column (see Callbacks for more details)
	editable.i				; (boolean, default.s ; false) determines if this data is editable by the user. will use the editor that matches the formatter by default. (see Manipulating Data for more details)
	editor.i					; set the editor to be used when editing the data. (see Manipulating Data for more details)
	visible.i					; (boolean, default.s ; true) determines if the column is visible. (see Column Visibility for more details)
	cssClass.s				; sets css classes on header and cells in this column. (value should be a string containing space seperated class names)
	tooltip.s					; sets the on hover tooltip for each cell in this column (see Formatting Data for more details)
	tooltipHeader.s		; sets the on hover tooltip for the column header (see Formatting Data for more details)
	editableTitle.s		; allows the user to edit the header titles. (see Editable Column Titles for more details)
	headerFilter.i		; filtering of columns from elements in the header (see Header Filtering for more details)
	mutator.s					; function for manipulating column values as they are parsed into the table (see Mutators for more details)
	mutateType.s			; defines whether mutation should occur on data parsing, cell editing or both (see Mutators for more details)
	accessor.s				; function to alter column values before they are read out the table using the getData function (see Accessors for more details)
EndStructure

#TabulatorGadgetSortString   = "string"   ; sorts column as strings of characters
#TabulatorGadgetSortNumber   = "number"		; sorts column as numbers (integer or float, will also handle numbers using "," seperators)
#TabulatorGadgetSortAlphaNum = "alphanum"	; sorts column as alpha numeric code
#TabulatorGadgetSortBoolean  = "boolean"	; sorts column as booleans
#TabulatorGadgetSortDate     = "date"			; sorts column as dates (for this you will need to set the date format using the dateFormat option when you create your table. default format is "dd/mm/yyyy")
#TabulatorGadgetSortTime     = "time"			; sorts column as time (formatted as hh:mm)

#TabulatorGadgetColumnAlignLeft   = "left"
#TabulatorGadgetColumnAlignRight  = "right"
#TabulatorGadgetColumnAlignCenter = "center"

Procedure TabulatorGadget(Gadget, x, y, Width, Height)
	
	!	var Gadget = cCommon.SetGadgetAndStuff(Gadget);
	
	!	var selector = $("<div />");
	
	!	selector.attr("id", "g_" + Gadget);
	
	! Width = Width - 1;
	! Height = Height - 1;
	
	!	cCommon.SetGadgetPosition(selector, x, y, Width, Height);
	
	!	selector.data("GadgetType", "TabulatorGadget");
	
	!	cCommon.CurrentContainer.append(selector);
	
	! selector.tabulator({
	!		height: Height + "px"
	!});
	
	! selector.tabulator("redraw", true);
	
	ProcedureReturn Gadget	
	
EndProcedure

; --------------------------------------

Procedure TabulatorGadget_SetGadgetAttribute(Gadget, Attribute, Value)
	
	Protected selector
	
	!	selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("SetGadgetAttribute");
	
	Protected Options
	
	Select Attribute
			
		Case #TabulatorGadgetData
			! selector.tabulator("setData", Value);
			
		Case #TabulatorGadgetColumns
			! selector.tabulator("setColumns", Value);
			
		Case #TabulatorGadgetOptions
			! var oldOptions = selector.tabulator("option");
			! var newOptions = {};
			! $.extend(true, newOptions, oldOptions, Value) 
			! selector.tabulator("option", newOptions);
			
		Case #TabulatorGadgetGroupBy
			! Options = { groupBy: Value };
			SetGadgetAttribute(Gadget, #TabulatorGadgetOptions, Options)
			
		Case #TabulatorGadgetFitColumns
			! Options = { fitColumns: Value };
			SetGadgetAttribute(Gadget, #TabulatorGadgetOptions, Options)
			
	EndSelect
	
	! selector.tabulator("redraw", true);
	
EndProcedure
RegisterUserGadgetProcedure("TabulatorGadget", "SetGadgetAttribute", @TabulatorGadget_SetGadgetAttribute())

Procedure TabulatorGadget_GetGadgetAttribute(Gadget, Attribute)
	
	!	var selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("GetGadgetAttribute");
	
	Select Attribute
		Case #TabulatorGadgetData
			! return selector.tabulator("getData");
		Case #TabulatorGadgetColumns
			! return selector.tabulator("getColumns");
		Case #TabulatorGadgetOptions
			! return selector.tabulator("option");
	EndSelect
	
EndProcedure
RegisterUserGadgetProcedure("TabulatorGadget", "GetGadgetAttribute", @TabulatorGadget_GetGadgetAttribute())

Procedure TabulatorGadget_ResizeGadget(Gadget, x, y, Width, Height)
	
	! var selector = $("#g_" + Gadget);
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("ResizeGadget");
	
	!	if (x !== cConstants.PB_Ignore) selector.css("left", x);
	!	if (y !== cConstants.PB_Ignore) selector.css("top", y);
	
	!	if (Width !== cConstants.PB_Ignore) { 
	!		Width = Width - 1;
	!		selector.css("width", Width);	
	! }
	
	!	if (Height !== cConstants.PB_Ignore) { 
	!		Height = Height - 1;
	!		selector.css("height", Height);	
	! };
	
	! selector.tabulator("redraw", true);
	
EndProcedure
RegisterUserGadgetProcedure("TabulatorGadget", "ResizeGadget", @TabulatorGadget_ResizeGadget())

Procedure TabulatorGadget_ClearGadgetItems(Gadget)
	
	!	selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("ClearGadgetItems");
	! selector.tabulator("clear");
	! selector.tabulator("redraw", true);
	
EndProcedure
RegisterUserGadgetProcedure("TabulatorGadget", "ClearGadgetItems", @TabulatorGadget_ClearGadgetItems())

Procedure TabulatorGadget_BindGadgetEvent(Gadget, Callback, EventType = #PB_All)
	
; 	If EventType = #PB_All
; 		EventType = #PB_EventType_LeftClick
; 	EndIf
; 	
; 	! var selector = $("#g_" + Gadget);
; 	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("BindGadgetEvent");
; 	!	var cb = Callback;
; 	!	if (cCommon.CheckFlag(EventType, cConstants.PB_EventType_LeftClick)) selector.data("onRowClick", cb);
	
EndProcedure
RegisterUserGadgetProcedure("TabulatorGadget", "BindGadgetEvent", @TabulatorGadget_BindGadgetEvent())


; IDE Options = PureBasic 5.50 (Windows - x86)
; CursorPosition = 62
; FirstLine = 9
; Folding = F9
; EnableXP